#include <stdlib.h>

#include "ble.h"
#include "ble_demo.h"
#include "ble_gattc.h"
#include "ble_srv_common.h"
#include "app_error.h"
#include "sdk_common.h"

uint32_t ro_charteric_add(ble_demo_service_t * p_demo);
uint32_t wo_charteric_add(ble_demo_service_t * p_demo);
uint32_t no_charteric_add(ble_demo_service_t * p_demo);
uint32_t speed_charteric_add(ble_demo_service_t * p_demo);

void on_write(ble_demo_service_t * p_demo, ble_evt_t * p_ble_evt)
{
    ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;

    if ( (p_evt_write->handle == p_demo->wo_charteric_handles.value_handle) &&
									(p_demo->data_handler != NULL) ) {
										
        p_demo->data_handler(p_demo, p_evt_write->data, p_evt_write->len);
    }
}


void demo_service_init(ble_demo_service_t *p_service, ble_demo_init_t *p_init){
    ble_uuid_t        service_uuid;
    ble_uuid128_t     base_uuid = DEMO_BASE_UUID;
    service_uuid.uuid = BLE_UUID_DEMO_SERVICE;

    APP_ERROR_CHECK(
		sd_ble_uuid_vs_add(&base_uuid, &service_uuid.type)
    );
    
    p_service->uuid_type = service_uuid.type;
	p_service->data_handler = p_init->data_handler;
	p_service->conn_handle = p_init->conn_handle;

	APP_ERROR_CHECK(
		sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY,
                                        &service_uuid,
                                        &p_service->service_handle)

	);
	
	APP_ERROR_CHECK(wo_charteric_add(p_service));
	APP_ERROR_CHECK(ro_charteric_add(p_service));
	//APP_ERROR_CHECK(no_charteric_add(p_service));
	APP_ERROR_CHECK(speed_charteric_add(p_service));
}

uint32_t wo_charteric_add(ble_demo_service_t * p_demo){
	ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.write         = 1;
    char_md.char_props.write_wo_resp = 1;
    char_md.p_char_user_desc         = NULL;
    char_md.p_char_pf                = NULL;
    char_md.p_user_desc_md           = NULL;
    char_md.p_cccd_md                = NULL;
    char_md.p_sccd_md                = NULL;

    ble_uuid.type = p_demo->uuid_type;
    ble_uuid.uuid = BLE_UUID_DEMO_WO_CHARACTERISTIC;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);

    attr_md.vloc    = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth = 0;
    attr_md.wr_auth = 0;
    attr_md.vlen    = 1;

	// Configure charteric value
    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid    = &ble_uuid;
    attr_char_value.p_attr_md = &attr_md;
    attr_char_value.init_len  = 0;
    attr_char_value.init_offs = 0;
    attr_char_value.max_len   = BLE_DEMO_MAX_DATA_LEN;

    return sd_ble_gatts_characteristic_add(p_demo->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_demo->wo_charteric_handles);
}

uint32_t ro_charteric_add(ble_demo_service_t * p_demo){
	ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
	
    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read = 1;
    char_md.p_char_user_desc         = NULL;
    char_md.p_char_pf                = NULL;
    char_md.p_user_desc_md           = NULL;
    char_md.p_cccd_md = NULL;
    char_md.p_sccd_md                = NULL;

    ble_uuid.type = p_demo->uuid_type;
    ble_uuid.uuid = BLE_UUID_DEMO_RO_CHARACTERISTIC;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    /*
     * BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS()
     * BLE_GAP_CONN_SEC_MODE_SET_OPEN()
     * BLE_GAP_CONN_SEC_MODE_SET_ENC_NO_MITM()
     */

    attr_md.vloc    = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth = 0;
    attr_md.wr_auth = 0;
    attr_md.vlen    = 1;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid    = &ble_uuid;
    attr_char_value.p_attr_md = &attr_md;
    attr_char_value.init_len  = 4;
    attr_char_value.init_offs = 0;
    attr_char_value.max_len   = BLE_DEMO_MAX_DATA_LEN;
    uint8_t value[4] = {0xB0,0xF0,0xC0,0xD0};
    attr_char_value.p_value = value;

    return sd_ble_gatts_characteristic_add(p_demo->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_demo->ro_charteric_handles);
}


uint32_t no_charteric_add(ble_demo_service_t * p_demo){
	ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
    ble_gatts_attr_md_t cccd_md;
	
	memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);

    cccd_md.vloc = BLE_GATTS_VLOC_STACK;
	
    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.notify = 1;
    char_md.p_char_user_desc         = NULL;
    char_md.p_char_pf                = NULL;
    char_md.p_user_desc_md           = NULL;
    char_md.p_cccd_md                = &cccd_md;
    char_md.p_sccd_md                = NULL;

    ble_uuid.type = p_demo->uuid_type;
    ble_uuid.uuid = BLE_UUID_DEMO_NO_CHARACTERISTIC;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);

    attr_md.vloc    = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth = 0;
    attr_md.wr_auth = 0;
    attr_md.vlen    = 1;

	// Configure charteric value
    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid    = &ble_uuid;
    attr_char_value.p_attr_md = &attr_md;
    attr_char_value.init_len  = sizeof(uint8_t);
    attr_char_value.init_offs = 0;
    attr_char_value.max_len   = BLE_DEMO_MAX_DATA_LEN;

    return sd_ble_gatts_characteristic_add(p_demo->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_demo->no_charteric_handles);
}

uint32_t speed_charteric_add(ble_demo_service_t * p_demo){
	ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
    ble_gatts_attr_md_t cccd_md;
	
	memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);

    cccd_md.vloc = BLE_GATTS_VLOC_STACK;
	
    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.notify = 1;
    char_md.p_char_user_desc         = NULL;
    char_md.p_char_pf                = NULL;
    char_md.p_user_desc_md           = NULL;
    char_md.p_cccd_md                = &cccd_md;
    char_md.p_sccd_md                = NULL;

    ble_uuid.type = p_demo->uuid_type;
    ble_uuid.uuid = BLE_UUID_DEMO_SPEED_CHARACTERISTIC;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);

    attr_md.vloc    = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth = 0;
    attr_md.wr_auth = 0;
    attr_md.vlen    = 1;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid    = &ble_uuid;
    attr_char_value.p_attr_md = &attr_md;
    attr_char_value.init_len  = sizeof(uint8_t);
    attr_char_value.init_offs = 0;
    attr_char_value.max_len   = BLE_DEMO_MAX_DATA_LEN;

    return sd_ble_gatts_characteristic_add(p_demo->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_demo->speed_charteric_handles);
}

void ble_demo_on_ble_evt(ble_demo_service_t * p_demo, ble_evt_t * p_ble_evt)
{
    if ((p_demo == NULL) || (p_ble_evt == NULL))
    {
        return;
    }

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GATTS_EVT_WRITE:
            on_write(p_demo, p_ble_evt);
            break;
        default:
            break;
    }
}

uint32_t ble_demo_ro_charteric_set(ble_demo_service_t * p_demo, uint8_t * p_data, uint16_t length)
{
    VERIFY_PARAM_NOT_NULL(p_demo);

    if (*p_demo->conn_handle == BLE_CONN_HANDLE_INVALID)
    {
        return NRF_ERROR_INVALID_STATE;
    }

    if (length > BLE_DEMO_MAX_DATA_LEN)
    {
        return NRF_ERROR_INVALID_PARAM;
    }

	ble_gatts_value_t value;
	value.len = length;
	value.offset = 0;
	value.p_value = p_data;

    return sd_ble_gatts_value_set(
		*p_demo->conn_handle,
		p_demo->ro_charteric_handles.value_handle,
		&value
	);
}


uint32_t ble_demo_charteric_send(ble_demo_service_t * p_demo, uint8_t * p_data, uint16_t length)
{
    ble_gatts_hvx_params_t hvx_params;

    VERIFY_PARAM_NOT_NULL(p_demo);

    if (*p_demo->conn_handle == BLE_CONN_HANDLE_INVALID)
    {
        return NRF_ERROR_INVALID_STATE;
    }

    if (length > BLE_DEMO_MAX_DATA_LEN)
    {
        return NRF_ERROR_INVALID_PARAM;
    }

    memset(&hvx_params, 0, sizeof(hvx_params));

    hvx_params.handle = p_demo->no_charteric_handles.value_handle;
    hvx_params.p_data = p_data;
    hvx_params.p_len  = &length;
    hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;

    return sd_ble_gatts_hvx(*p_demo->conn_handle, &hvx_params);
}


uint32_t ble_demo_speed_send(ble_demo_service_t * p_demo, uint8_t * p_data, uint16_t length)
{
    ble_gatts_hvx_params_t hvx_params;

    VERIFY_PARAM_NOT_NULL(p_demo);

    if (*p_demo->conn_handle == BLE_CONN_HANDLE_INVALID)
    {
        return NRF_ERROR_INVALID_STATE;
    }

    if (length > BLE_DEMO_MAX_DATA_LEN)
    {
        return NRF_ERROR_INVALID_PARAM;
    }

    memset(&hvx_params, 0, sizeof(hvx_params));

    hvx_params.handle = p_demo->speed_charteric_handles.value_handle;
    hvx_params.p_data = p_data;
    hvx_params.p_len  = &length;
    hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;

    return sd_ble_gatts_hvx(*p_demo->conn_handle, &hvx_params);
}
