/*
 * Copyright (c) 2012 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic Semiconductor. The use,
 * copying, transfer or disclosure of such information is prohibited except by express written
 * agreement with Nordic Semiconductor.
 *
 */



#ifndef BLE_DEMO_C_H__
#define BLE_DEMO_C_H__

#include <stdint.h>
#include <stdbool.h>
#include "ble.h"
#include "ble_gatt.h"
#include "ble_db_discovery.h"

#define DEMO_BASE_UUID                  {{0xBB, 0x11, 0x66, 0xBB, 0x00, 0x00, 0xBB, 0x55, 0x93, 0xF3, 0xA3, 0xB5, 0x00, 0x00, 0x40, 0x6E}}
#define BLE_UUID_DEMO_SERVICE           0x0001
#define BLE_UUID_DEMO_WO_CHARACTERISTIC 0x0002
#define BLE_UUID_DEMO_RO_CHARACTERISTIC 0x0003
#define BLE_UUID_DEMO_NO_CHARACTERISTIC 0x0004
#define BLE_UUID_DEMO_SPEED_CHARACTERISTIC 0x0005

#define BLE_DEMO_MAX_DATA_LEN 16

typedef struct ble_demo_service_s ble_demo_service_t;

typedef struct ble_demo_init_s ble_demo_init_t;

typedef void (*ble_demo_data_handler_t) (ble_demo_service_t * p_demo, uint8_t * p_data, uint16_t length);

struct ble_demo_service_s {
	uint8_t uuid_type;
	uint16_t service_handle;
	uint16_t *conn_handle;
	ble_gatts_char_handles_t ro_charteric_handles;
	ble_gatts_char_handles_t no_charteric_handles;
	ble_gatts_char_handles_t wo_charteric_handles;
	ble_gatts_char_handles_t speed_charteric_handles;
	ble_demo_data_handler_t data_handler;
};

struct ble_demo_init_s {
	uint16_t *conn_handle;
	ble_demo_data_handler_t data_handler;
};

void demo_service_init(ble_demo_service_t *p_service, ble_demo_init_t *p_init);

void ble_demo_on_ble_evt(ble_demo_service_t * p_demo, ble_evt_t * p_ble_evt);

uint32_t ble_demo_charteric_send(ble_demo_service_t * p_demo, uint8_t * p_data, uint16_t length);

uint32_t ble_demo_ro_charteric_set(ble_demo_service_t * p_demo, uint8_t * p_data, uint16_t length);

uint32_t ble_demo_speed_send(ble_demo_service_t * p_demo, uint8_t * p_data, uint16_t length);

#endif // BLE_DEMO_C_H__

/** @} */
